@@ -14,7 +14,7 @@ binmode STDERR, ':utf8';
 use open qw(:std :encoding(UTF-8));
 
 use Carp qw(carp croak);
-use Config::Abstraction 0.34;
+use Config::Abstraction 0.36;
 use Data::Dumper;
 use Data::Section::Simple;
 use File::Basename qw(basename);
@@ -26,12 +26,16 @@ use Exporter 'import';
 
 our @EXPORT_OK = qw(generate);
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 =head1 NAME
 
 App::Test::Generator - Generate fuzz and corpus-driven test harnesses
 
+=head1 VERSION
+
+Version 0.08
+
 =head1 SYNOPSIS
 
 From the command line:
@@ -192,10 +196,13 @@ Recognized items:
 
 When using named parameters
 
-	our %input = (
-		name => { type => 'string', optional => 0 },
-		age => { type => 'integer', optional => 1 },
-	);
+  input:
+    name:
+      type: string
+      optional: false
+    age:
+      type: integer
+      optional: true
 
 Supported basic types used by the fuzzer: C<string>, C<integer>, C<number>, C<boolean>, C<arrayref>, C<hashref>.
 (You can add more types; they will default to C<undef> unless extended.)
@@ -242,6 +249,13 @@ C<My-Widget.conf> -> C<My::Widget>.
 
 To ensure new is called with no arguments, you still need to define new, thus:
 
+  module: MyModule
+  function: my_function
+
+  new:
+
+For the legacy Perl variable syntax, use the empty string:
+
   our $new = '';
 
 =item * C<%cases> - optional Perl static corpus, when the output is a simple string (expected => [ args... ]):
@@ -520,7 +534,9 @@ sub generate
 
 			$module = $config->{module} if(exists($config->{module}));
 			$function = $config->{function} if(exists($config->{function}));
-			$new = $config->{new} if(exists($config->{new}));
+			if(exists($config->{new})) {
+				$new = defined($config->{'new'}) ? $config->{new} : '_UNDEF';
+			}
 			$yaml_cases = $config->{yaml_cases} if(exists($config->{yaml_cases}));
 			$seed = $config->{seed} if(exists($config->{seed}));
 			$iterations = $config->{iterations} if(exists($config->{iterations}));
@@ -631,10 +647,16 @@ sub generate
 	sub _validate_config {
 		my $config = $_[0];
 
-		for my $key('module', 'function', 'input') {
+		for my $key('module', 'function') {
 			croak "Missing required '$key' specification" unless $config->{$key};
 		}
-		croak 'Invalid input specification' unless(ref $config->{input} eq 'HASH');
+		if((!defined($config->{'input'})) && (!defined($config->{'output'}))) {
+			# Routine takes no input and no output, so there's nothing that would be gained using this software
+			croak('You must specify at least one of input and output');
+		}
+		if($config->{'input'}) {
+			croak('Invalid input specification') unless(ref $config->{input} eq 'HASH');
+		}
 
 		# Validate types, constraints, etc.
 		for my $param (keys %{$config->{input}}) {
@@ -1120,6 +1142,9 @@ sub fuzz_inputs {
 					# Data::Random
 					push @cases, { _input => rand_set(set => $input{'memberof'}, size => 1) }
 				} else {
+					if((!defined($input{'min'})) || ($input{'min'} >= 1)) {
+						push @cases, { _input => '0' } if(!defined($input{'memberof'}));
+					}
 					push @cases, { _input => 'hello', _STATUS => 'DIES' };
 				}
 				push @cases, { _input => '' } if((!exists($input{'min'})) || ($input{'min'} == 0));
@@ -1234,10 +1259,17 @@ sub fuzz_inputs {
 				# --- matches (regex) ---
 				if (defined $spec->{matches}) {
 					my $regex = $spec->{matches};
-					push @cases, { $field => 'match123' } if "match123" =~ $regex;
+					push @cases, { $field => 'match123' } if 'match123' =~ $regex;
 					push @cases, { $field => 'nope', _STATUS => 'DIES' } unless 'nope' =~ $regex;
 				}
 
+				# --- nomatch (regex) ---
+				if (defined $spec->{nomatch}) {
+					my $regex = $spec->{nomatch};
+					push @cases, { $field => 'match123' } if "match123" !~ $regex;
+					push @cases, { $field => 'nope', _STATUS => 'DIES' } unless 'nope' !~ $regex;
+				}
+
 				# --- memberof ---
 				if (defined $spec->{memberof}) {
 					my @set = @{ $spec->{memberof} };
@@ -1352,7 +1384,11 @@ sub fuzz_inputs {
 		push @cases, { '_STATUS' => 'DIES' };	# At least one argument is needed
 	}
 
-	push @cases, { '_STATUS' => 'DIES', map { $_ => undef } keys %input } if($config{'test_undef'});
+	if(scalar keys %input) {
+		push @cases, { '_STATUS' => 'DIES', map { $_ => undef } keys %input } if($config{'test_undef'});
+	} else {
+		push @cases, { };	# Takes no input
+	}
 
 	# If it's not in mandatory_strings it sets to 'undef' which is the idea, to test { value => undef } in the args
 	push @cases, { map { $_ => $mandatory_strings{$_} } keys %input, %mandatory_objects } if($config{'test_undef'});
@@ -1437,6 +1473,24 @@ sub fuzz_inputs {
 					push @cases, { _input => undef, _STATUS => 'DIES' } if($config{'test_undef'});
 					push @cases, { _input => "\0", _STATUS => 'DIES' } if($config{'test_nuls'});
 				}
+				if(defined $input{nomatch}) {
+					my $re = $input{nomatch};
+
+					# --- Positive controls ---
+					foreach my $val (@candidate_good) {
+						if ($val !~ $re) {
+							push @cases, { _input => $val };
+							last; # one good match is enough
+						}
+					}
+
+					# --- Negative controls ---
+					foreach my $val (@candidate_bad) {
+						if ($val =~ $re) {
+							push @cases, { _input => $val, _STATUS => 'DIES' };
+						}
+					}
+				}
 			} elsif ($type eq 'arrayref') {
 				if (defined $input{min}) {
 					my $len = $input{min};
@@ -1562,9 +1616,9 @@ sub fuzz_inputs {
 								for my $count ($len - 1, $len, $len + 1) {
 									my $str = rand_char() x $count;
 									if($str =~ $re) {
-										push @cases, { %mandatory_strings, ( $field => $str ) };
+										push @cases, { %mandatory_strings, ( $field => $str ), _LINE => __LINE__ };
 									} else {
-										push @cases, { %mandatory_strings, ( $field => $str, _STATUS => 'DIES' ) };
+										push @cases, { %mandatory_strings, ( $field => $str, _STATUS => 'DIES', _LINE => __LINE__ ) };
 									}
 								}
 							} else {
@@ -1594,6 +1648,24 @@ sub fuzz_inputs {
 						push @cases, { $field => undef, _STATUS => 'DIES' } if($config{'test_undef'});
 						push @cases, { $field => "\0", _STATUS => 'DIES' } if($config{'test_nuls'});
 					}
+					if(defined $spec->{nomatch}) {
+						my $re = $spec->{nomatch};
+
+						# --- Positive controls ---
+						foreach my $val (@candidate_good) {
+							if ($val !~ $re) {
+								push @cases, { $field => $val };
+								last; # one good match is enough
+							}
+						}
+
+						# --- Negative controls ---
+						foreach my $val (@candidate_bad) {
+							if ($val =~ $re) {
+								push @cases, { $field => $val, _STATUS => 'DIES' };
+							}
+						}
+					}
 				} elsif ($type eq 'arrayref') {
 					if (defined $spec->{min}) {
 						my $len = $spec->{min};
@@ -1706,7 +1778,11 @@ foreach my $case (@{fuzz_inputs()}) {
 		$mess = "[% function %] %s";
 	}
 
-	if(my $status = delete $case->{'_STATUS'} || delete $output{'_STATUS'}) {
+	if(my $line = (delete $case->{'_LINE'} || delete $output{'_LINE'})) {
+		diag("Test case from line number $line") if($ENV{'TEST_VERBOSE'});
+	}
+
+	if(my $status = (delete $case->{'_STATUS'} || delete $output{'_STATUS'})) {
 		if($status eq 'DIES') {
 			dies_ok { [% call_code %] } sprintf($mess, 'dies');
 		} elsif($status eq 'WARNS') {
